@@ -50,18 +50,6 @@ def add_diff_columns(df, name, agg_dict, agg_type, diff_d=[0.3, 0.5, 0.7]):
 #         extra_agg_dict[f'J_bh_{n}'] = 'first'
 
 
-# def rsi_indicator(df, back_hour_list, need_shift, extra_agg_dict={}, add_diff=False):
-#     # --- RSI ---  在期货市场很有效
-#     close_dif = df['close'].diff()
-#     df['up'] = np.where(close_dif > 0, close_dif, 0)
-#     df['down'] = np.where(close_dif < 0, abs(close_dif), 0)
-#     for n in back_hour_list:
-#         a = df['up'].rolling(n).sum()
-#         b = df['down'].rolling(n).sum()
-#         df[f'RSI_bh_{n}'] = (a / (a + b)).shift(1 if need_shift else 0)  # RSI
-#         extra_agg_dict[f'RSI_bh_{n}'] = 'first'
-
-
 # def avg_price_indicator(df, back_hour_list, need_shift, extra_agg_dict={}, add_diff=False):
 #     # --- 均价 ---  对应低价股策略(预计没什么用)
 #     # 策略改进思路：以下所有用到收盘价的因子，都可尝试使用均价代替
@@ -155,6 +143,23 @@ def add_diff_columns(df, name, agg_dict, agg_type, diff_d=[0.3, 0.5, 0.7]):
 #         df[f'量价相关系数_bh_{n}'] = df['close'].rolling(n).corr(df['quote_volume']).shift(1 if need_shift else 0)
 #         extra_agg_dict[f'量价相关系数_bh_{n}'] = 'first'
 
+def rsi_indicator(df, back_hour_list, need_shift, extra_agg_dict={}, add_diff=False):
+    # --- RSI ---  在期货市场很有效
+    close_dif = df['close'].diff()
+    df['up'] = np.where(close_dif > 0, close_dif, 0)
+    df['down'] = np.where(close_dif < 0, abs(close_dif), 0)
+    for n in back_hour_list:
+        a = df['up'].rolling(n).sum()
+        b = df['down'].rolling(n).sum()
+
+        f_name = f'RSI_bh_{n}'
+        df[f_name] = (a / (a + b)).shift(1 if need_shift else 0)  # RSI
+        extra_agg_dict[f_name] = 'first'
+        if type(add_diff) is list:
+            add_diff_columns(df, f_name, extra_agg_dict, 'first', diff_d=add_diff)
+        elif add_diff:
+            add_diff_columns(df, f_name, extra_agg_dict, 'first')
+
 
 def bias_indicator(df, back_hour_list, need_shift, extra_agg_dict={}, add_diff=False):
     # --- bias ---  涨跌幅更好的表达方式 bias 币价偏离均线的比例。
@@ -185,6 +190,50 @@ def cci_indicator(df, back_hour_list, need_shift, extra_agg_dict={}, add_diff=Fa
             add_diff_columns(df, f_name, extra_agg_dict, 'first')
 
 
+def cci_ema_indicator(df, back_hour_list, need_shift, extra_agg_dict={}, add_diff=False):
+    # magic_cci
+    for n in back_hour_list:
+        """
+        N=14
+        TP=(HIGH+LOW+CLOSE)/3
+        MA=MA(TP,N)
+        MD=MA(ABS(TP-MA),N)
+        CCI=(TP-MA)/(0.015MD)
+        CCI 指标用来衡量典型价格（最高价、最低价和收盘价的均值）与其
+        一段时间的移动平均的偏离程度。CCI 可以用来反映市场的超买超卖
+        状态。一般认为，CCI 超过 100 则市场处于超买状态；CCI 低于-100
+        则市场处于超卖状态。当 CCI 下穿 100/上穿-100 时，说明股价可能
+        要开始发生反转，可以考虑卖出/买入。
+        """
+        df['oma'] = df['open'].ewm(span=n, adjust=False).mean()  # 取 open 的ema
+        df['hma'] = df['high'].ewm(span=n, adjust=False).mean()  # 取 high 的ema
+        df['lma'] = df['low'].ewm(span=n, adjust=False).mean()  # 取 low的ema
+        df['cma'] = df['close'].ewm(span=n, adjust=False).mean()  # 取 close的ema
+        df['tp'] = (df['oma'] + df['hma'] + df['lma'] + df[
+            'cma']) / 4  # 魔改CCI基础指标 将TP=(HIGH+LOW+CLOSE)/3  替换成以open/high/low/close的ema 的均值
+        df['ma'] = df['tp'].ewm(span=n, adjust=False).mean()  # MA(TP,N)  将移动平均改成 ema
+        df['abs_diff_close'] = abs(df['tp'] - df['ma'])  # ABS(TP-MA)
+        df['md'] = df['abs_diff_close'].ewm(span=n, adjust=False).mean()  # MD=MA(ABS(TP-MA),N)  将移动平均替换成ema
+
+        f_name = f'cci_ema_bh_{n}'
+        df[f_name] = (df['tp'] - df['ma']) / df['md']  # CCI=(TP-MA)/(0.015MD)  CCI在一定范围内
+        df[f_name] = df[f_name].shift(1 if need_shift else 0)  # 取前一周期防止未来函数  实盘中不需要
+        extra_agg_dict[f_name] = 'first'
+        if type(add_diff) is list:
+            add_diff_columns(df, f_name, extra_agg_dict, 'first', diff_d=add_diff)
+        elif add_diff:
+            add_diff_columns(df, f_name, extra_agg_dict, 'first')
+        # # 删除中间数据
+        del df['oma']
+        del df['hma']
+        del df['lma']
+        del df['cma']
+        del df['tp']
+        del df['ma']
+        del df['abs_diff_close']
+        del df['md']
+
+
 def psy_indicator(df, back_hour_list, need_shift, extra_agg_dict={}, add_diff=False):
     # --- psy ---  量价相关选股策略
     for n in back_hour_list:
@@ -908,3 +957,211 @@ def rwil_indicator(df, back_hour_list, need_shift, extra_agg_dict={}, add_diff=F
         del df['TR']
         del df['ATR']
         del df['RWIL']
+
+
+def ma_displaced_indicator(df, back_hour_list, need_shift, extra_agg_dict={}, add_diff=False):
+    # ma_displaced 2*n
+    for n in back_hour_list:
+        """
+        N=20
+        M=10
+        MA_CLOSE=MA(CLOSE,N)
+        MADisplaced=REF(MA_CLOSE,M)
+        MADisplaced 指标把简单移动平均线向前移动了 M 个交易日，用法
+        与一般的移动平均线一样。如果收盘价上穿/下穿 MADisplaced 则产
+        生买入/卖出信号。
+        有点变种bias
+        """
+        ma = df['close'].rolling(2 * n, min_periods=1).mean()  # MA(CLOSE,N) 固定俩个参数之间的关系  减少参数
+        ref = ma.shift(n)  # MADisplaced=REF(MA_CLOSE,M)
+
+        f_column = f'ma_displaced_bh_{n}'
+        df[f_column] = df['close'] / ref - 1  # 去量纲
+        df[f_column] = df[f_column].shift(1 if need_shift else 0)
+        extra_agg_dict[f_column] = 'first'
+        # 差分
+        if type(add_diff) is list:
+            add_diff_columns(df, f_column, extra_agg_dict, 'first', diff_d=add_diff)
+        elif add_diff:
+            add_diff_columns(df, f_column, extra_agg_dict, 'first')
+
+
+def dbcd_indicator(df, back_hour_list, need_shift, extra_agg_dict={}, add_diff=False):
+    # DBCD 6*n
+    for n in back_hour_list:
+        df['ma'] = df['close'].rolling(n, min_periods=1).mean()  # MA(CLOSE,N)
+        df['BIAS'] = (df['close'] - df['ma']) / df['ma'] * 100  # BIAS=(CLOSE-MA(CLOSE,N)/MA(CLOSE,N))*100
+        df['BIAS_DIF'] = df['BIAS'] - df['BIAS'].shift(3 * n)  # BIAS_DIF=BIAS-REF(BIAS,M)
+        df['DBCD'] = df['BIAS_DIF'].rolling(3 * n + 2, min_periods=1).mean()
+        # df['dbcd'] = df['BIAS_DIF'].ewm(span=3 * n3).mean()  # DBCD=SMA(BIAS_DIFF,T,1)
+        f_name = f'dbcd_bh_{n}'
+        df[f_name] = df['DBCD'].shift(1 if need_shift else 0)
+        extra_agg_dict[f_name] = 'first'
+
+        if type(add_diff) is list:
+            add_diff_columns(df, f_name, extra_agg_dict, 'first', diff_d=add_diff)
+        elif add_diff:
+            add_diff_columns(df, f_name, extra_agg_dict, 'first')
+
+        del df['ma']
+        del df['BIAS']
+        del df['BIAS_DIF']
+        del df['DBCD']
+
+
+def uos_indicator(df, back_hour_list, need_shift, extra_agg_dict={}, add_diff=False):
+    # UOS 指标
+    for n in back_hour_list:
+        M = n
+        N = 2 * n
+        O = 4 * n
+        df['ref_close'] = df['close'].shift(1)
+        df['TH'] = df[['high', 'ref_close']].max(axis=1)
+        df['TL'] = df[['low', 'ref_close']].min(axis=1)
+        df['TR'] = df['TH'] - df['TL']
+        df['XR'] = df['close'] - df['TL']
+        df['XRM'] = df['XR'].rolling(M).sum() / df['TR'].rolling(M).sum()
+        df['XRN'] = df['XR'].rolling(N).sum() / df['TR'].rolling(N).sum()
+        df['XRO'] = df['XR'].rolling(O).sum() / df['TR'].rolling(O).sum()
+        df['UOS'] = 100 * (df['XRM'] * N * O + df['XRN'] * M * O + df['XRO'] * M * N) / (M * N + M * O + N * O)
+
+        f_name = f'uos_bh_{n}'
+        df[f_name] = df['UOS'].shift(1 if need_shift else 0)
+        extra_agg_dict[f_name] = 'first'
+        if type(add_diff) is list:
+            add_diff_columns(df, f_name, extra_agg_dict, 'first', diff_d=add_diff)
+        elif add_diff:
+            add_diff_columns(df, f_name, extra_agg_dict, 'first')
+        del df['ref_close']
+        del df['TH']
+        del df['TL']
+        del df['TR']
+        del df['XR']
+        del df['XRM']
+        del df['XRN']
+        del df['XRO']
+        del df['UOS']
+
+
+def trix_indicator(df, back_hour_list, need_shift, extra_agg_dict={}, add_diff=False):
+    # TRIX 3n
+    for n in back_hour_list:
+        df['ema'] = df['close'].ewm(n, adjust=False).mean()
+        df['ema_ema'] = df['ema'].ewm(n, adjust=False).mean()
+        df['ema_ema_ema'] = df['ema_ema'].ewm(n, adjust=False).mean()
+
+        df['TRIX'] = (df['ema_ema_ema'] - df['ema_ema_ema'].shift(1)) / df['ema_ema_ema'].shift(1)
+
+        f_name = f'trix_bh_{n}'
+        df[f_name] = df['TRIX'].shift(1 if need_shift else 0)
+        extra_agg_dict[f_name] = 'first'
+        if type(add_diff) is list:
+            add_diff_columns(df, f_name, extra_agg_dict, 'first', diff_d=add_diff)
+        elif add_diff:
+            add_diff_columns(df, f_name, extra_agg_dict, 'first')
+        del df['ema']
+        del df['ema_ema']
+        del df['ema_ema_ema']
+        del df['TRIX']
+
+
+def vwap_bias_indicator(df, back_hour_list, need_shift, extra_agg_dict={}, add_diff=False):
+    # bias因子以均价表示
+    for n in back_hour_list:
+        df['vwap'] = df['volume'] / df['quote_volume']
+        ma = df['vwap'].rolling(n, min_periods=1).mean()
+        f_name = f'vwap_bias_bh_{n}'
+        df[f_name] = df['vwap'] / ma - 1
+        df[f_name] = df[f_name].shift(1)
+        extra_agg_dict[f_name] = 'first'
+        if type(add_diff) is list:
+            add_diff_columns(df, f_name, extra_agg_dict, 'first', diff_d=add_diff)
+        elif add_diff:
+            add_diff_columns(df, f_name, extra_agg_dict, 'first')
+        del df['vwap']
+
+
+def ko_indicator(df, back_hour_list, need_shift, extra_agg_dict={}, add_diff=False):
+    # KO
+    for n in back_hour_list:
+        df['price'] = (df['high'] + df['low'] + df['close']) / 3
+        df['V'] = np.where(df['price'] > df['price'].shift(1), df['volume'], -df['volume'])
+        df['V_ema1'] = df['V'].ewm(n, adjust=False).mean()
+        df['V_ema2'] = df['V'].ewm(int(n * 1.618), adjust=False).mean()
+        df['KO'] = df['V_ema1'] - df['V_ema2']
+        # 标准化
+        f_name = f'ko_bh_{n}'
+        df[f_name] = (df['KO'] - df['KO'].rolling(n).min()) / (
+            df['KO'].rolling(n).max() - df['KO'].rolling(n).min())
+        df[f_name] = df[f_name].shift(1)
+        extra_agg_dict[f_name] = 'first'
+        if type(add_diff) is list:
+            add_diff_columns(df, f_name, extra_agg_dict, 'first', diff_d=add_diff)
+        elif add_diff:
+            add_diff_columns(df, f_name, extra_agg_dict, 'first')
+        del df['price']
+        del df['V']
+        del df['V_ema1']
+        del df['V_ema2']
+        del df['KO']
+
+    # df['comp_zack'] = df['reg_diff_0.5'] * (df['uos_diff_0.3'] + df['k_diff_0.3']) \
+    #     + df['pmo'] * (df['trix_diff_0.5'] + df['vwap_bias_diff_0.3']) \
+    #     + df['dbcd_diff_0.5'] * (df['dc'] + df['ko'])
+    # return df
+
+
+def mtm_mean_indicator(df, back_hour_list, need_shift, extra_agg_dict={}, add_diff=False):
+    # mtm_mean
+    for n in back_hour_list:
+        f_name = f'mtm_mean_bh_{n}'
+        df[f_name] = (df['close'] / df['close'].shift(n) - 1).rolling(window=n, min_periods=1).mean().shift(1 if need_shift else 0)
+        extra_agg_dict[f_name] = 'first'
+        if type(add_diff) is list:
+            add_diff_columns(df, f_name, extra_agg_dict, 'first', diff_d=add_diff)
+        elif add_diff:
+            add_diff_columns(df, f_name, extra_agg_dict, 'first')
+
+
+def force_indicator(df, back_hour_list, need_shift, extra_agg_dict={}, add_diff=False):
+    # force
+    for n in back_hour_list:
+        df['force'] = df['quote_volume'] * (df['close'] - df['close'].shift(1))
+
+        f_name = f'force_bh_{n}'
+        df[f_name] = df['force'].rolling(n, min_periods=1).mean()
+        df[f_name] = df[f_name].shift(1 if need_shift else 0)
+        extra_agg_dict[f_name] = 'first'
+
+        if type(add_diff) is list:
+            add_diff_columns(df, f_name, extra_agg_dict, 'first', diff_d=add_diff)
+        elif add_diff:
+            add_diff_columns(df, f_name, extra_agg_dict, 'first')
+        # 删除中间过程数据
+        del df['force']
+
+
+def bolling_indicator(df, back_hour_list, need_shift, extra_agg_dict={}, add_diff=False):
+    # Bolling
+    for n in back_hour_list:
+        # 计算布林上下轨
+        df['std'] = df['close'].rolling(n, min_periods=1).std()
+        df['ma'] = df['close'].rolling(n, min_periods=1).mean()
+        df['upper'] = df['ma'] + 1.0 * df['std']
+        df['lower'] = df['ma'] - 1.0 * df['std']
+        # 将上下轨中间的部分设为0
+        condition_0 = (df['close'] <= df['upper']) & (df['close'] >= df['lower'])
+        condition_1 = df['close'] > df['upper']
+        condition_2 = df['close'] < df['lower']
+        df.loc[condition_0, 'distance'] = 0
+        df.loc[condition_1, 'distance'] = df['close'] - df['upper']
+        df.loc[condition_2, 'distance'] = df['close'] - df['lower']
+
+        f_name = f'bolling_bh_{n}'
+        df[f_name] = df['distance'] / df['std']
+        df[f_name] = df[f_name].shift(1 if need_shift else 0)
+        extra_agg_dict[f_name] = 'first'
+        if type(add_diff) is list:
+            add_diff_columns(df, f_name, extra_agg_dict, 'first', diff_d=add_diff)
+        elif add_diff:
+            add_diff_columns(df, f_name, extra_agg_dict, 'first')
