@@ -2,43 +2,6 @@ from .base import *
 from cy_widgets.neutral.neutral_indicators import *
 
 
-class BinanceNeutralCCIGAPStrategy(NeutralStrategyBase):
-
-    @classmethod
-    def strategy_with_parameters(cls, parameters):
-        """初始化"""
-        return BinanceNeutralCCIGAPStrategy(int(parameters[0]), f'{int(parameters[1])}h', float(parameters[2]))
-
-    @property
-    def display_name(self):
-        return "BinanceNeutralCCIGAPStrategy"
-
-    @property
-    def candle_count_4_cal_factor(self):
-        return 48 * 3 * 2 + 10
-
-    def cal_factor(self, df):
-        # -- 48H CCI Factor--
-        n = 48
-        oma = ta.WMA(df.open, n)
-        hma = ta.WMA(df.high, n)
-        lma = ta.WMA(df.low, n)
-        cma = ta.WMA(df.close, n)
-        tp = (hma + lma + cma + oma) / 4
-        ma = ta.WMA(tp, n)
-        md = ta.WMA(abs(cma - ma), n)
-
-        df[f'CCI_bh_{n}'] = (tp - ma) / md
-
-        # ---- GapTrue ----
-        ma = df['close'].rolling(window=n, min_periods=1).mean()
-        gap = cma - ma
-        df[f'GapTrue_bh_{n}'] = gap / abs(gap).rolling(window=n).sum()
-
-        df['factor'] = - 9 * df['CCI_bh_48'] + 77 * df['GapTrue_bh_48']
-        return df
-
-
 class BinanceNeutralPMOGAPREGStrategy(NeutralStrategyBase):
     # ('pmo', True, 3, 1), ('gap', True, 24, 0.4), ('reg', False, 12, 0.7)
 
@@ -49,7 +12,7 @@ class BinanceNeutralPMOGAPREGStrategy(NeutralStrategyBase):
 
     @property
     def display_name(self):
-        return "1.PMO_GAP_REG"
+        return "1.PMO_GAP_REG_6H"
 
     @property
     def candle_count_4_cal_factor(self):
@@ -91,7 +54,7 @@ class BinanceNeutralStrategy_REG_RCCD_VIDYA_RWI_APZ(NeutralStrategyBase):
 
     @property
     def display_name(self):
-        return "2.REG_RCCD_VIDYA_RWI_APZ"
+        return "2.REG_RCCD_VIDYA_RWI_APZ_4H"
 
     @property
     def candle_count_4_cal_factor(self):
@@ -132,7 +95,7 @@ class BinanceNeutralStrategy_V1UP_REG_PMO(NeutralStrategyBase):
 
     @property
     def display_name(self):
-        return "3.V1UP_REG_PMO"
+        return "3.V1UP_REG_PMO_6H"
 
     @property
     def candle_count_4_cal_factor(self):
@@ -171,3 +134,53 @@ class BinanceNeutralStrategy_V1UP_REG_PMO(NeutralStrategyBase):
         """ 横截面计算步骤 """
         for (f_name, _, _, _, f_weight) in self.factor_configs:
             df['factor'] += df.groupby('s_time')[f_name + '_factor'].rank(method='first') * f_weight
+
+
+class BinanceNeutralStrategy_CCIEMA_MTMMEAN(NeutralStrategyBase):
+    # ('cci_ema', 1, 24, 0, 0.4), ('mtm_mean', 0, 3, 0.3, 1)
+
+    @classmethod
+    def strategy_with_parameters(cls, parameters):
+        """初始化"""
+        return BinanceNeutralStrategy_CCIEMA_MTMMEAN(int(parameters[0]), f'{int(parameters[1])}h', float(parameters[2]))
+
+    @property
+    def display_name(self):
+        return "4.CCIEMA_MTMMEAN_4H"
+
+    @property
+    def candle_count_4_cal_factor(self):
+        return 105
+
+    @property
+    def factor_configs(self):
+        return ('cci_ema', 1, 24, 0, 0.4), ('mtm_mean', 0, 3, 0.3, 1)
+
+    def cal_factor(self, df):
+        for (f_name, f_reverse, f_bh, f_diff, _) in self.factor_configs:
+            # 加差分
+            if f_diff > 0:
+                add_diff = [f_diff]
+                f_column = f'{f_name}_bh_{f_bh}_diff_{f_diff}'
+            else:
+                add_diff = False
+                f_column = f'{f_name}_bh_{f_bh}'
+            # 计算因子
+            eval(f'{f_name}_indicator')(df, [f_bh], False, add_diff=add_diff)
+            # 设置到 factor
+            if f_reverse:
+                df[f'{f_name}_factor'] = -df[f_column]
+            else:
+                df[f'{f_name}_factor'] = df[f_column]
+        # 横截面这里没有 factor，先给0
+        df['factor'] = 0
+        return df
+
+    def update_agg_dict(self, agg_dict):
+        for (f_name, _, _, _, _) in self.factor_configs:
+            agg_dict[f'{f_name}_factor'] = 'last'
+
+    def cal_compound_factors(self, df):
+        """ 横截面计算步骤 """
+        for (f_name, _, _, _, f_weight) in self.factor_configs:
+            df['factor'] += df.groupby('s_time')[f_name + '_factor'].rank(method='first') * f_weight
